package zap

import (
	"sync"
	"testing"
	"time"

	"git.perx.ru/perxis/perxis-go/logs"
	logmocks "git.perx.ru/perxis/perxis-go/logs/mocks"
	"github.com/stretchr/testify/assert"
	"github.com/stretchr/testify/mock"
	"github.com/stretchr/testify/require"
)

func TestBufferedWriteSyncer_Write(t *testing.T) {
	service := logmocks.NewService(t)
	service.On("Log", mock.Anything, mock.Anything).
		Return(nil).
		Run(func(args mock.Arguments) {
			entries := args.Get(1).([]*logs.Entry)
			require.Equal(t, 2, len(entries))
		}).
		Once()

	ws := &BufferedWriteSyncer{Service: service}

	err := ws.Write(&logs.Entry{Message: "first log message"})
	require.NoError(t, err)

	err = ws.Write(&logs.Entry{Message: "second log message"})
	require.NoError(t, err)

	err = ws.Stop()
	require.NoError(t, err)

}

func TestBufferedWriteSyncer_Write_Concurrent(t *testing.T) {
	service := logmocks.NewService(t)
	service.On("Log", mock.Anything, mock.Anything).
		Return(nil).
		Run(func(args mock.Arguments) {
			entries := args.Get(1).([]*logs.Entry)
			require.Equal(t, 100, len(entries))
		}).
		Once()

	ws := &BufferedWriteSyncer{Service: service}

	var wg sync.WaitGroup
	for i := 0; i < 100; i++ {
		wg.Add(1)
		go func(wg *sync.WaitGroup) {
			defer wg.Done()
			require.NoError(t, ws.Write(&logs.Entry{Message: "log message"}))
		}(&wg)
	}

	wg.Wait()
	require.NoError(t, ws.Stop())

}

func TestBufferedWriteSyncer_Flush(t *testing.T) {
	service := logmocks.NewService(t)
	service.On("Log", mock.Anything, mock.Anything).
		Return(nil).
		Run(func(args mock.Arguments) {
			entries := args.Get(1).([]*logs.Entry)
			require.Equal(t, 10, len(entries))
		}).
		Times(10)

	ws := &BufferedWriteSyncer{Service: service}

	for i := 0; i < 10; i++ {
		for j := 0; j < 10; j++ {
			require.NoError(t, ws.Write(&logs.Entry{Message: "log message"}))
		}
		require.NoError(t, ws.Sync())
	}

	require.NoError(t, ws.Stop())

}

func TestBufferedWriteSyncer_MaxBufferSize(t *testing.T) {
	service := logmocks.NewService(t)
	service.On("Log", mock.Anything, mock.Anything).
		Return(nil).
		Run(func(args mock.Arguments) {
			entries := args.Get(1).([]*logs.Entry)
			assert.Equal(t, 10, len(entries))
		}).
		Times(10)

	ws := &BufferedWriteSyncer{Service: service, MaxBufferSize: 10}

	for i := 0; i < 100; i++ {
		require.NoError(t, ws.Write(&logs.Entry{Message: "log message"}))
	}

	require.NoError(t, ws.Stop())

}

func TestBufferedWriteSyncer_FlushInterval(t *testing.T) {
	service := logmocks.NewService(t)
	service.On("Log", mock.Anything, mock.Anything).
		Return(nil).
		Run(func(args mock.Arguments) {
			entries := args.Get(1).([]*logs.Entry)
			assert.Equal(t, 10, len(entries))
		}).
		Once()

	ws := &BufferedWriteSyncer{Service: service, FlushInterval: time.Second}

	for j := 0; j < 10; j++ {
		require.NoError(t, ws.Write(&logs.Entry{Message: "log message"}))
	}

	time.Sleep(3 * time.Second) // ждем, пока сработает интервал
	require.NoError(t, ws.Stop())

}
