package extension

import (
	"testing"

	"go.uber.org/zap"

	"go.uber.org/zap/zaptest/observer"

	"git.perx.ru/perxis/perxis-go/pkg/collections"
	"git.perx.ru/perxis/perxis-go/pkg/schema"
	"git.perx.ru/perxis/perxis-go/pkg/schema/field"
	"git.perx.ru/perxis/perxis-go/pkg/setup"
	"github.com/stretchr/testify/assert"
)

func Test_isMetadataEqual(t *testing.T) {
	tests := []struct {
		name string
		s1   *schema.Schema
		s2   *schema.Schema
		want bool
	}{
		{
			"Not equal #1 (no metadata)",
			schema.New("name", field.String()).WithMetadata(MetadataKey, "test"),
			schema.New("name", field.String()),
			false,
		},
		{
			"Not equal #2 (different metadata)",
			schema.New("name", field.String()).WithMetadata(MetadataKey, "test"),
			schema.New("name", field.String()).WithMetadata("test", "test"),
			false,
		},
		{
			"Equal #1 (no metadata)",
			schema.New("name", field.String()),
			schema.New("name", field.String()),
			true,
		},
		{
			"Equal #2 (equal metadata)",
			schema.New("name", field.String()).WithMetadata(MetadataKey, "test"),
			schema.New("name", field.String()).WithMetadata(MetadataKey, "test"),
			true,
		},
	}
	for _, tt := range tests {
		t.Run(tt.name, func(t *testing.T) {
			assert.Equalf(t, tt.want, isSameExtension(tt.s1, tt.s2), "isMetadataExtensionEqual(%v, %v)", tt.s1, tt.s2)
		})
	}
}

func TestDefaultUpdate_ExtensionCollections(t *testing.T) {
	tests := []struct {
		name       string
		exist      *collections.Collection
		collection *collections.Collection
		force      bool
		update     bool
		wantLogs   int
	}{
		{
			name: "Same extension",
			exist: &collections.Collection{ID: "coll", SpaceID: "sp", EnvID: "env",
				Schema: schema.New("name", field.String()).WithMetadata("extension", "extension-1")},
			collection: &collections.Collection{ID: "coll", SpaceID: "sp", EnvID: "env", Name: "new name",
				Schema: schema.New("name", field.String()).WithMetadata("extension", "extension-1")},
			update: true,
		},
		{
			name: "Other extension",
			exist: &collections.Collection{ID: "coll", SpaceID: "sp", EnvID: "env",
				Schema: schema.New("name", field.String()).WithMetadata("extension", "extension-1")},
			collection: &collections.Collection{ID: "coll", SpaceID: "sp", EnvID: "env", Name: "new name",
				Schema: schema.New("name", field.String()).WithMetadata("extension", "extension-2")},
			update:   false,
			wantLogs: 1,
		},
		{
			name: "Other extension force",
			exist: &collections.Collection{ID: "coll", SpaceID: "sp", EnvID: "env",
				Schema: schema.New("name", field.String()).WithMetadata("extension", "extension-1")},
			collection: &collections.Collection{ID: "coll", SpaceID: "sp", EnvID: "env", Name: "new name",
				Schema: schema.New("name", field.String()).WithMetadata("extension", "extension-2")},
			force:  true,
			update: true,
		},
		{
			name: "User collection",
			exist: &collections.Collection{ID: "coll", SpaceID: "sp", EnvID: "env",
				Schema: schema.New("name", field.String())},
			collection: &collections.Collection{ID: "coll", SpaceID: "sp", EnvID: "env", Name: "new name",
				Schema: schema.New("name", field.String()).WithMetadata("extension", "extension-1")},
			update:   false,
			wantLogs: 1,
		},
		{
			name: "User collection force",
			exist: &collections.Collection{ID: "coll", SpaceID: "sp", EnvID: "env",
				Schema: schema.New("name", field.String())},
			collection: &collections.Collection{ID: "coll", SpaceID: "sp", EnvID: "env", Name: "new name",
				Schema: schema.New("name", field.String()).WithMetadata("extension", "extension-1")},
			force:  true,
			update: true,
		},
	}

	for _, tt := range tests {
		t.Run(tt.name, func(t *testing.T) {
			cfg := setup.NewConfig()
			cfg.Collections.Add(tt.collection, UpdateExtensionCollections())

			core, ob := observer.New(zap.DebugLevel)
			logger := zap.New(core)
			set := setup.NewSetup(nil, "sp", "env", logger).WithForce(tt.force)
			_, update := cfg.Collections[0].UpdateFunc(set, tt.exist, tt.collection)

			assert.Equal(t, tt.update, update)
			logs := ob.TakeAll()
			assert.Len(t, logs, tt.wantLogs)

		})
	}
}
